/* ----------------------------------------------------------------------------
 *         ATMEL Microcontroller Software Support  -  ROUSSET  -
 * ----------------------------------------------------------------------------
 * Copyright (c) 2006, Atmel Corporation

 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer below.
 *
 * - Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the disclaimer below in the documentation and/or
 * other materials provided with the distribution.
 *
 * Atmel's name may not be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * DISCLAIMER: THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
 * DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ----------------------------------------------------------------------------
 * File Name           : main.c
 * Object              :
 * Creation            : ODi Apr 19th 2006
 *-----------------------------------------------------------------------------
 */
#include "include/part.h"
#include "include/main.h"
#include "include/debug.h"
#include "include/dataflash.h"
#include "include/nandflash.h"
#include "include/norflash.h"

//------------------------------------------------------------------------------
// Defines and Types
//------------------------------------------------------------------------------
// This address where is wrote the bootloader type (communication between 
// Firstboot and eboot)
#define BOOTLOADER_FLASH_CONFIG_ADDR			 (AT91C_IRAM + 0x0)

// Flash type depending of logical base address
#define LOGIC_ADDR_MASK							0xF0000000
#define DATAFLASH_CS0_LOGIC_BASE_ADDR					0xC0000000
#define DATAFLASH_CS1_LOGIC_BASE_ADDR					0xD0000000
#define NANDFLASH_CS3_LOGIC_BASE_ADDR					0xE0000000


#define DATAFLASH_BOOTLOADER_SETTINGS_LOGICAL_OFFSET	0x00035000
#define DATAFLASH_BOOTLOADER_CODE_LOGICAL_OFFSET	0x00005000
#define DATAFLASH_IMAGE_FLASH_OFFSET			0x00020000

#define NANDFLASH_BOOTLOADER_SETTINGS_LOGICAL_OFFSET	0x00060000
#define NANDFLASH_BOOTLOADER_CODE_LOGICAL_OFFSET	0x00020000

// Location of the windows CE image in the nandflash
#define NANDFLASH_IMAGE_FLASH_OFFSET					0x00080000
#define EBOOT_FLASH_CODE_ADDR		NANDFLASH_CS3_LOGIC_BASE_ADDR | NANDFLASH_BOOTLOADER_CODE_LOGICAL_OFFSET
#define EBOOT_FLASH_SETTINGS_ADDR	NANDFLASH_CS3_LOGIC_BASE_ADDR | NANDFLASH_BOOTLOADER_SETTINGS_LOGICAL_OFFSET


//------------------------------------------------------------------------------
// EBoot Param	!!
//------------------------------------------------------------------------------
#define EBOOT_VERSION_MAJOR             2
#define EBOOT_VERSION_MINOR             0

unsigned long ComputeCRC32(unsigned char *buffer, int len);



#define DWORD unsigned long
#define UCHAR unsigned char

#pragma pack(1)
// The EBOOT_CFG structure holds a variety of configuration parameters.
// When adding new parameters, make sure that the size of the structure in bytes is 
// an integral number of DWORDS.  Pad the structure if necessary.
typedef struct
{
	DWORD dwSize;
	DWORD dwEbootCodeFlashAddr;
	DWORD dwEbootSettingsFlashAddr;
	DWORD dwCrc32;

} T_BOOTLOADER_FLASH_CONFIG;
#pragma pack()

//-----------------------------------------------------------------------------
//! \fn			unsigned long ComputeCRC32(unsigned char *buffer, int len)
//!
//! \brief		This function compute a crc 32 from a pointer for a certain length
//!
//! \param		buffer Pointer on memory you want compute a crc
//! \param  		len Length of zone which you want compute a crc
//!
//! \return  		The CRC32 wanted
//-----------------------------------------------------------------------------
unsigned long ComputeCRC32(unsigned char *buffer, int len)
{
  unsigned long crc;  
  unsigned long temp;
  int j;
 
  crc = 0xFFFFFFFF;
 
  while (len--) 
  {
    temp = (unsigned long)((crc & 0xFF) ^ *buffer++);
    for (j = 0; j < 8; j++) 
    {
      if (temp & 0x1)
        temp = (temp >> 1) ^ 0xEDB88320;
      else
        temp >>= 1;
    }
    crc = (crc >> 8) ^ temp;
  }
  return crc ^ 0xFFFFFFFF;
}

/*------------------------------------------------------------------------------*/
/* Function Name       : main							*/
/* Object              : Main function						*/
/* Input Parameters    : none							*/
/* Output Parameters   : True							*/
/*------------------------------------------------------------------------------*/
int main(void)
{
/* ================== 1st step: Hardware Initialization ================= */
	/* Performs the hardware initialization */
       
#ifdef CFG_HW_INIT
	hw_init();
#endif
	
#ifdef CFG_DEBUG_MAIN
	dbg_print("dopo HW INIT...\n\r");
#endif /* CFG_DEBUG_MAIN */


/* ==================== 2nd step: Load from media ==================== */
	/* Load from Dataflash in RAM */
#ifdef CFG_DATAFLASH
	load_df(AT91C_SPI_PCS_DATAFLASH, IMG_ADDRESS, IMG_SIZE, JUMP_ADDR);
#endif


	/* Load from Nandflash in RAM */
#ifdef CFG_NANDFLASH

#ifdef CFG_DEBUG_MAIN
        char tmp[32]={0,0};
        dbg_print("prima load_nand...\n\r");
        if(IMG_SIZE == 0xC0000)
            dbg_print("\tIMG_SIZE correct 0xC0000\n\r");
#endif /* CFG_DEBUG */
	/* cancella la SDRAM */
        memset(JUMP_ADDR, 0x00, IMG_SIZE);

#ifdef CFG_DEBUG_MAIN
        dbg_print("prima load_nand...\n\r");
#endif /* CFG_DEBUG_MAIN */

	load_nandflash(IMG_ADDRESS, IMG_SIZE, JUMP_ADDR);

#ifdef CFG_DEBUG_MAIN
	dbg_print("dopo load_nand...\n\r");
#endif /* CFG_DEBUG_MAIN */
#endif


	/* Load from Norflash in RAM */
#ifdef CFG_NORFLASH
        load_norflash(IMG_ADDRESS_NOR, IMG_SIZE_NOR, JUMP_ADDR);  
#endif

#if 1
	volatile T_BOOTLOADER_FLASH_CONFIG *pBootFlashConfig = (T_BOOTLOADER_FLASH_CONFIG*)(BOOTLOADER_FLASH_CONFIG_ADDR);
	
	dbg_print("FIRSTBOOT_StartBoot int \n\r");
	// Set flash configuration
	pBootFlashConfig->dwEbootCodeFlashAddr = EBOOT_FLASH_CODE_ADDR;
	pBootFlashConfig->dwEbootSettingsFlashAddr = EBOOT_FLASH_SETTINGS_ADDR;
	pBootFlashConfig->dwSize = sizeof(T_BOOTLOADER_FLASH_CONFIG);
	pBootFlashConfig->dwCrc32 = ComputeCRC32((UCHAR*)pBootFlashConfig, sizeof(T_BOOTLOADER_FLASH_CONFIG)-sizeof(DWORD));
#endif
/* ==================== 3rd step:  Process the Image =================== */
	/* Uncompress the image */
#ifdef GUNZIP
	decompress_image((void *)IMG_ADDRESS, (void *)JUMP_ADDR, IMG_SIZE);	/* NOT IMPLEMENTED YET */
#endif /* GUNZIP */

/* ==================== 4th step: Start the application =================== */
	/* Set linux arguments */
#ifdef LINUX_ARG
	linux_arg(LINUX_ARG);	/* NOT IMPLEMENTED YET */
#endif /* LINUX_ARG */
#ifdef CFG_DEBUG_MAIN
	dbg_print("prima jump...\n\r");
#endif /* CFG_DEBUG_MAIN */

	/* Jump to the Image Address */
	return JUMP_ADDR;
}

